package asd.td8;

import asd.graphe.*;

/**
 * EXERCICE 16 - TD 8
 * Une classe pour mettre  jour un ACM aprs l'ajout d'une nouvelle
 * arete au graphe.
 */
public class ACMajout {

    private GrapheNonOrienteValue G; // le graphe
    private GrapheNonOrienteValue T; // son ACM

    private boolean[] marque; // pour le parcours de maxCycle

    /**
     * Fabrique un objet ACMajout avec le graphe G et son ACM T.
     */
    public ACMajout(GrapheNonOrienteValue G, GrapheNonOrienteValue T) {
        this.G = G;
        this.T = T;
        this.marque = new boolean[T.ordre()];
    }

    /**
     * Ajoute l'arete uv au graphe G et mets l'arbre ACM  jour.
     */
    public void ajouter(AreteValuee uv) {
        int u = uv.origine();
        int v = uv.extremite();
        int w = uv.valuation();

        Arete max = maxCycle(u,v);

        int um = max.origine();
        int vm = max.extremite();
        int wm = G.valuation(um,vm);

        if ( w < wm ) {
            T.deconnecter(um,vm);
            T.connecter(u,v,w);
        }
    }

    /**
     * Retourne l'arete de poids maximum parmi toutes les aretes
     * de l'UNIQUE chemin entre u et v dans ACM.
     */
    private Arete maxCycle(int u, int v) {
        int[] areteMax = new int[3];
        areteMax[2] = Integer.MIN_VALUE;
        maxCycle(u,v,areteMax);
        return new Arete(areteMax[0],areteMax[1]);
    }

    /**
     * Retourne l'arete de poids maximum parmi toutes les aretes
     * de l'UNIQUE chemin entre i et v dans ACM. L'arete est
     * retourne sous la forme d'un tableau A  3 lments avec
     * A[0] et A[1] les extrmits de l'arete et A[2] sa valuation.
     */
    private boolean maxCycle(int i, int v, int[] areteMax) {
        marque[i] = true;
        Adjacents adj = T.adjacents(i);
        while ( adj.nonVide() ) {
            int s = adj.suivant();
            if ( s == v ) {
                maxArete(i,v,areteMax);
                return true;
            }
            else
                if ( ! marque[s] )
                    if ( maxCycle(s,v,areteMax) ) {
                        if ( G.valuation(i,s) > valuation(areteMax) )
                            maxArete(i,s,areteMax);
                        return true;
                    }
        }
        return false;
    }

    /**
     * Pour rendre le code de la mthode maxCycle plus clair !
     */
    private int valuation(int[] areteMax) {
        return areteMax[2];
    }

    /**
     * Pour rendre le code de la mthode maxCycle plus clair !
     */
    private void maxArete(int i, int j, int[] areteMax) {
        areteMax[0] = i; areteMax[1] = j; areteMax[2] = G.valuation(i,j);
    }
}
