package asd.td8;

import asd.graphe.*;

/**
 * EXERCICE 17 - TD 8
 * Une classe pour la mise  jour d'un ACM aprs la suppression d'une arete.
 */
public class ACMsuppression {

    private GrapheNonOrienteValue G; // le graphe
    private GrapheNonOrienteValue T; // l'ACM du graphe

    private int[] c; // la couleur des sommets
    private static final int rien   = 0; // le sommet n'a jamais t visit
    private static final int arbre  = 1; // le sommet a t visit dans l'arbre
    private static final int graphe = 2; // le sommet a t visit dans le graphe

    /**
     * Construit un objet ACMsuppression pour le graphe G et son ACM T.
     */
    public ACMsuppression(GrapheNonOrienteValue G, GrapheNonOrienteValue T) {
        this.G = G;
        this.T = T;
        this.c = new int[T.ordre()];
    }

    /**
     * Supprime l'arete uv dans le graphe G et mets  jour T.
     */
    public void supprimer(Arete uv) {
        int u = uv.origine();
        int v = uv.extremite();
        G.deconnecter(u,v);

        if ( dansT(u,v) ) {        // (u,v) est dans T
            T.deconnecter(u,v);    // on l'enlve et on dconnecte T
            visiterT(u);
            Arete min = visiterG(u); // min est l'arete traversante de poids min
            int um = min.origine();
            int vm = min.extremite();
            T.connecter(um,vm,G.valuation(um,vm));
        }
    }

    /**
     * Teste si l'arete (u,v) est dans T en numrant
     * les sommets adjacents  u dans T.
     */
    private boolean dansT(int u, int v) {
        Adjacents adj = T.adjacents(u);
        while ( adj.nonVide() )
            if ( adj.suivant() == v )
                return true;
        return false;
    }

    /**
     * Visite l'arbre T en profondeur  partir du sommet i
     * et marque les sommets visits avec 'arbre'.
     */
    private void visiterT(int i) {
        c[i] = arbre;
        Adjacents adj = G.adjacents(i);
        while ( adj.nonVide() ) {
            int s = adj.suivant();
            if ( c[s] == rien )
                visiterT(s);
        }
    }

    /**
     * Visite le graphe G en profondeur   partir du sommet i
     * et retourne l'arete de poids minimum qui traverse S'.
     */
    private Arete visiterG(int i) {
        int[] areteMin = new int[3];
        areteMin[2] = Integer.MAX_VALUE;
        visiter(i,areteMin);
        return new Arete(areteMin[0],areteMin[1]);
    }
        
    /**
     * Visite le graphe G en profondeur  partir du sommet i
     * et place dans areteMin l'arete de poids minimum qui traverse S'.
     */
    private void visiter(int i, int[] areteMin) {
        c[i] = graphe;
        Adjacents adj = G.adjacents(i);
        while ( adj.nonVide() ) {
            int s = adj.suivant();
            if ( c[s] == rien ) {
                if ( G.valuation(i,s) < areteMin[2] ) {
                    areteMin[0] = i; areteMin[1] = s;
                    areteMin[2] = G.valuation(i,s);
                }
            }
            else
                if ( c[s] == arbre )
                    visiter(s,areteMin);
        }
    }
}
