package asd.td8;

import java.util.Vector;
import asd.graphe.Adjacents;
import asd.graphe.GrapheOriente;
import asd.base.PileStatique;

/**
 * EXERCICE 8 - TD 8
 * Une classe pour dterminer une base d'un graphe orient.
 */
public class Base {

    private GrapheOriente G; // le graphe orient
    private boolean[] blanc; // blanc[u] <==> le sommet u est blanc

    /**
     * Construit un objet Base pour la recherche d'une base de G.
     * Paramtre G : un graphe orient.
     * Complexit : O(1).
     */
    public Base(GrapheOriente G) {
	this.G = G;
	this.blanc = new boolean[G.ordre()];
    }

    ////////// deuxime parcours du graphe

    /**
     * Retourne un ensemble de sommets sous la forme d'un tableau base
     * de boolen qui forment une base du graphe.
     * Complexit : O(S + A) o S et A sont les nombres de sommets et
     * d'arcs du graphe.
     */
    public boolean[] base() { // parcourir2
        boolean[] base = new boolean[G.ordre()];
        PileStatique pile = parcourir1();

        for ( int i = 0; i < G.ordre(); i++ ) {
            blanc[i] = true;
        }
        // tous les sommets sont blancs

        while ( nonVide(pile) ) {
            int i = depiler(pile);
	    if ( blanc[i] ) {
                base[i] = true;
		visiter(i);
	    }
	}
        return base;
    }

    ////////// premier parcours du graphe

    private PileStatique parcourir1() {
        PileStatique pile = new PileStatique(G.ordre());

        for ( int i = 0; i < G.ordre(); i++ ) {
            blanc[i] = true;
        }
        // tous les sommets sont blancs

        for ( int i = 0; i < G.ordre(); i++ ) {
            if ( blanc[i] ) {
                empiler(i,pile); // on empile que les racines
                visiter(i);      // des arbres de G_pi
            }
        }
        return pile;
    }

    ////////// visite en profondeur commune aux deux parcours

    private void visiter(int i) {
        blanc[i] = false;
        Adjacents adj = G.adjacents(i);
        
        while ( adj.nonVide() ) {
            int s = adj.suivant();
            if ( blanc[s] ) {
                visiter(s);
            }
        }
    }

    ////////// pour manipuler un pile d'int plus simplement

    private void empiler(int i, PileStatique pile) {
        pile.empiler(new Integer(i));
    }

    private int depiler(PileStatique pile) {
        int i = ((Integer) pile.sommet()).intValue();
        pile.depiler();
        return i;
    }

    private boolean nonVide(PileStatique pile) {
        return ! pile.vide();
    }
}
