package asd.td8;

import asd.base.ObjetAClef;
import asd.tri.TasCroissant;
import asd.graphe.*;

/**
 * EXERCICE 14 - TD 8
 * Une classe pour l'algorithme de Kruskal sur un graphe
 * non ncessairement connexe.
 */
public class KruskalNonConnexe {

    private GrapheNonOrienteValue G; // la FCM  construire

    // attributs du parcours en profondeur

    private int[] p; // p[j] = i <==> i est le pre de j
    private int[] c; // la couleur des sommets

    private static final int blanc = 0;
    private static final int gris  = 1;
    private static final int noir  = 2;

    /**
     * Construit un objet KruskalNonConnexe pour calculer un arbre couvrant de
     * poids minimum du graphe non orient valu G.
     * Complexit : O(1).
     */
    public KruskalNonConnexe(GrapheNonOrienteValue G) {
	this.G = G;
    }

    /**
     * Retourne une foret couvrante de poids minimum du graphe G.
     * Complexit : O(n.lg(n)) o n est le nombre d'artes de G.
     */
    public GrapheNonOrienteValue fcm() {

        GrapheNonOrienteValue FCM = new GrapheNonOrienteValue(G.ordre());
        TasCroissant T = new TasCroissant(G.aretes());
        initialiser(T);

	Composante CC = new Composante(G.ordre()); // un sommet par cc

	while ( T.taille() > 0 ) { // initialement  "ncc > 1"

            Arete uv = (Arete) T.extraireMinimum().info();

	    int u = uv.origine();
	    int v = uv.extremite();

	    if ( CC.separes(u,v) ) {
		FCM.connecter(u,v,G.valuation(u,v));
		CC.reunir(u,v); 
	    }
	}
        // INVARIANT : 
        // - le graphe FCM est un sous-FCM de G.
        // - u et v sont dans la mme cc <==> u <--> v dans FCM
	// VARIANT :
        // |T| dcrot et est minor par 0
        //
	return FCM;
    }

    /**
     * Effectue un parcours en profondeur du graphe G et ajoute chaque
     * arte de G une et une seule fois dans le tas T.
     * Complexit : O(S + A) o S est l'ordre du graphe et A le nombre
     * d'artes.
     */
    private void initialiser(TasCroissant T) {
        c = new int[G.ordre()];
        p = new int[G.ordre()];
	for ( int i = 0; i < G.ordre(); i++ ) {
	    c[i] = blanc;
	    p[i] = -1;
	}
	for ( int i = 0; i < G.ordre(); i++ ) // initialement "visiter(0,T)"
	    if ( c[i] == blanc )
                visiter(i,T);
    }

    /**
     * Visite le sous-graphe de sommet i en profondeur et ajoute chaque arte
     * visite une et une seule fois dans le tas T.
     * Prcondition : i est blanc.
     * Postrelation : i est noir et chaque arte visite a t ajoute une et
     * une seule fois dans le tas T.
     * Complexit : O(Ai) o Ai est le nombre d'artes accessibles depuis i.
     */
    private void visiter(int i, TasCroissant T) {
	c[i] = gris;
	Adjacents adj = G.adjacents(i);
	
	while ( adj.nonVide() ) {
	    int s = adj.suivant();

	    if ( c[s] == blanc ) {
                T.ajouter(new ObjetAClef(new Arete(i,s),
                                         G.valuation(i,s)));
		p[s] = i;
		visiter(s,T);
	    }
            else {
                if ( c[s] == gris && s != p[i] ) {
                    T.ajouter(new ObjetAClef(new Arete(i,s),
                                             G.valuation(i,s)));
                }
            }
	}
	c[i] = noir;
    }
}
