package asd.td8;

import asd.graphe.GrapheOriente;
import asd.graphe.Adjacents;

/**
 * EXERCICE 5 - TD 8
 * Une classe pour le parcours en profondeur d'un graphe orient qui affiche
 * le type exact de chaque arc.
 */
public class PPOriente {

    private GrapheOriente G;
    
    private int[] a; // le numro de l'arbre dans la fort en profondeur

    private int[] c; // la couleur des sommets

    private static final int blanc = 0;
    private static final int gris  = 1;
    private static final int noir  = 2;

    public PPOriente(GrapheOriente G) {
	this.G = G;
	this.a = new int[G.ordre()];
	this.c = new int[G.ordre()];
    }

    /**
     * Effectue un parcours en profondeur du graphe et affiche le type de
     * tous les arcs.
     * Complexit : O(S + A) o S est l'ordre du graphe et A le nombre d'arcs.
     */
    public void parcourir() {
        int k = 0; // pour numroter les arbres
	
	for ( int i = 0; i < G.ordre(); i++ ) {
	    c[i] = blanc;
	}
        // tous les sommets sont blancs et aucun sommet n'a de pre

	for ( int i = 0; i < G.ordre(); i++ ) {
	    if ( c[i] == blanc ) {
		visiter(i,k++);
	    }
	}
    }

    /**
     * Visite le sous-graphe de sommet i en profondeur et affiche le type de
     * tous les arcs.
     * Prcondition : i est blanc.
     * Postrelation : i est noir et le type de tous les arcs accessibles depuis
     * i a t affich.
     * Complexit : O(Ai) o Ai est le nombre d'arcs accessibles depuis i.
     */
    private void visiter(int i, int k) {
        a[i] = k;
	c[i] = gris;
	Adjacents adj = G.adjacents(i);
	
	while ( adj.nonVide() ) {
	    int s = adj.suivant();

            // dbut de tratement
	    switch ( c[s] ) {
            case blanc :
                afficher(i,s,"de liaison");
		visiter(s,k);
                break;
            case gris :
                afficher(i,s,"retour");
                break;
            case noir :
                if ( a[i] == a[s] ) {
                    afficher(i,s,"couvrant");
                }
                else {
                    afficher(i,s,"crois");
                }
            }
            // fin de tratement
	}
	c[i] = noir;
    }

    /**
     * Pour afficher le type de l'arc (i,j)
     */
    private void afficher(int i, int j, String s) {
        System.out.println(i + " --> " + j + " est un arc " + s);
    }
}
